/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/zephyr.h>
#include <logging/log.h>
#include <zephyr/irq.h>
#include <nrf52833.h>
#include "nrfx_twim.h"
#include <drivers/i2c.h>

const nrfx_twim_t m_twim_instance       = NRFX_TWIM_INSTANCE(0);

#define MPU_TWI_BUFFER_SIZE             14
#define MPU_TWI_TIMEOUT                 10000
volatile static bool twi_xfer_done      = false;
uint8_t twi_tx_buffer[MPU_TWI_BUFFER_SIZE];

#define LOG_MODULE_NAME app
LOG_MODULE_REGISTER(LOG_MODULE_NAME);

// void my_twim_handler(nrfx_twim_evt_t const * p_event, void * p_context)
// {
//     // LOG_INF("TWIM callback");
//     switch(p_event->type)
//     {
//         case NRFX_TWIM_EVT_DONE:
//             twi_xfer_done = true;   // This is the event we are waiting for.
//             break;
//         case NRFX_TWIM_EVT_ADDRESS_NACK:
//             LOG_ERR("address nack");
//             break;
//         case NRFX_TWIM_EVT_DATA_NACK:
//             LOG_ERR("data nack");
//             break;
//         case NRFX_TWIM_EVT_OVERRUN:
//             LOG_ERR("overrun");
//             break;
//         case NRFX_TWIM_EVT_BUS_ERROR:
//             LOG_ERR("bus error");
//             break;
//         default:
//             LOG_ERR("default (should never happen)");
//             break;
//     }
// }

int twi_init(void)
{
    // Setup peripheral interrupt.
    IRQ_CONNECT(DT_IRQN(DT_NODELABEL(i2c0)),DT_IRQ(DT_NODELABEL(i2c0), priority), nrfx_isr, nrfx_twim_0_irq_handler,0);
    irq_enable(DT_IRQN(DT_NODELABEL(i2c0)));

    int err = 0;

    const nrfx_twim_config_t twim_config = {                          \
        .scl                = 4,                                      \
        .sda                = 3,                                      \
        .frequency          = NRF_TWIM_FREQ_400K,                     \
        .interrupt_priority = NRFX_TWIM_DEFAULT_CONFIG_IRQ_PRIORITY,  \
        .hold_bus_uninit    = false,                                  \
    };
    err = nrfx_twim_init(&m_twim_instance,
                          &twim_config,
                          NULL,
                          NULL);
                          
    if (err != NRFX_SUCCESS) {
        LOG_ERR("twim_init failed. (err %x)", err);
        return err;
    }

    nrfx_twim_enable(&m_twim_instance);
    
    return 0;
}

int i2c_rx(nrfx_twim_t const *        p_instance,
           uint8_t                    address,
           uint8_t *                  p_data,
           uint8_t                    length)
{
    int err;
    nrfx_twim_xfer_desc_t xfer = NRFX_TWIM_XFER_DESC_RX(address, p_data, length);
    err = nrfx_twim_xfer(p_instance, &xfer, 0);
    if (err != NRFX_SUCCESS) {
        LOG_ERR("err %08x, addr %02x", err, address);
    }
    else {
        LOG_INF("err %08x, addr %02x FOUND DEVICE", err, address);
    }
    return err;
    
}

void main(void)
{
    int err;
	LOG_INF("Hello World! %s", CONFIG_BOARD);
    uint8_t msg[1] = {0x00};

    err = twi_init();
    if (err){
        LOG_ERR("twi_init() returned %d", err);
    }
    for (uint8_t i=0; i<0x7F; i++) {
        i2c_rx(&m_twim_instance, i, msg, 1);
        k_sleep(K_MSEC(10));
    }

    for (;;) {
        LOG_INF("1 sec passed");
        k_sleep(K_MSEC(1000));
    }
}
